#include "chainprotein.h"
#include "reader.h"
#include <sstream>
#include <assert.h>

/**
 * \brief Protein::Protein()
 * Constructeur d'une Proteine
 */
ChainProtein::ChainProtein()
{
    chain_id = ' ';
    atomeLourd = 0;
}


/**
 * \brief Protein::process_line()
 * Permet de lire les ligne du fichier pdb qui concerne une proteine (ligne ATOM-TER)
 * Cette fonction crée les ATOME et les ajoute a la proteine en cour de lecture
 */
void ChainProtein::process_line(const char *line)
{
    Line_type typeLigne= typedeLigne(line);
    if (typeLigne == TER)
    {

    }
    else if (typeLigne == ATOM || typeLigne == HETATM )
    {
        // the read values are not zero padded so we must fill the buffers for strings with 0s.
        bool hetatm = false;
        int snum=-1;
        char name[5]={'\0','\0','\0','\0','\0',};
        char alt='\0';
        char resname[4]={'\0','\0','\0','\0'};
        char chain;
        int resnum=-1;
        char insertion_residue_code;
        float x,y,z;
        float occupancy, tempFactor;
        char segID[5]={'\0','\0','\0','\0','\0'};
        char element[3]={'\0','\0','\0'};
        char charge[3]={'\0','\0','\0'};

        if(typeLigne == ATOM)
        {
            sscanf(line, atom_line_iformat_,
            &snum, name, &alt, resname, &chain, &resnum, &insertion_residue_code,
            &x,&y,&z, &occupancy, &tempFactor, segID, element, charge);
        }
        else
        {
            hetatm = true;
            sscanf(line, hetatm_line_iformat_,
            &snum, name, &alt, resname, &chain, &resnum, &insertion_residue_code,
            &x,&y,&z, &occupancy, &tempFactor, segID, element, charge);
        }

        if(element[0] != 'H' && element[1] != 'H' && element[2] != 'H')
        {
            this->atomeLourd++;
        }

        if (chain_id==' ')
            chain_id=chain;

        if (!(chain_id==' ' || chain== chain_id))
        {
            std::ostringstream oss;
            oss << "Confusion over chain numbers. Expected " << chain_id << " got " << chain << " on line:\n" << line;
            return;
        }

        if (resnum < 0)
        {
            std::ostringstream oss;
            oss << "Got negative residue index on line " << line;
            return;
        }

        Atom a(hetatm,snum,name,alt,resname,chain,resnum,insertion_residue_code,Point(x,y,z),occupancy,tempFactor,segID,element,charge);
        Atome.push_back(a);


    }
    else
    {
        assert(0);
    }
}
void ChainProtein::process_coord(double x, double y, double z)
{
    Atom a(Point(x,y,z));
    Atome.push_back(a);
    this->atomeLourd++;
}

/**
 * \brief Protein::getNbAtome()
 * Permet de recuperer le nombre d'atome ATOME de la proteine
 * \return Le nombre d'atome
 */
unsigned int ChainProtein::getNbAtome()
{
    return this->Atome.size();
}
/**
 * \brief Protein::getNbAtomeLourd()
 * Permet de recuperer le nombre d'atome ATOME de la proteine
 * \return Le nombre d'atome
 */
unsigned int ChainProtein::getNbAtomeLourd()
{
    return this->atomeLourd;
}

/**
 * \brief Protein::atomeAt()
 * Permet de recuperer une référence vers un atome contenue dans la proteine
 * \param i Numero de l'atome
 * \return Une reference vers l'atome
 */
Atom& ChainProtein::atomeAt(int i)
{
    return this->Atome.at(i);
}

/**
 * \brief Protein::getchainID()
 * Renvoie l'id de la chaine de proteine
 * \return l'ID de la proteine(PDB)
 */
char ChainProtein::getchainID()
{
    return this->chain_id;
}

/**
 * \brief Protein::write()
 * Ecrit la proteine au format PDB dans le stream de sortit
 * \param out Flux de sortit vers un fichier ou autre
 */
void ChainProtein::write(std::ostream &out)
{
    char line[81];

    for (unsigned int i = 0; i < this->Atome.size(); i++)
    {
        this->Atome.at(i).write(out);
    }
    static const char terformat[]="TER   %5d      %3s %c";
    if (!this->Atome.empty())
    {

        sprintf(line, terformat, this->Atome.back().getIndex()+1, this->Atome.back().getresname(), this->Atome.back().getchain());
        out << line << std::endl;
    }
}
